/*
* SACD Decoder plugin
* Copyright (c) 2011-2016 Maxim V.Anisiutkin <maxim.anisiutkin@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with FFmpeg; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#ifndef _DSD_STREAM_SERVICE_H_INCLUDED
#define _DSD_STREAM_SERVICE_H_INCLUDED

#include <foobar2000.h>

class dsd_chunk_t {
	t_samplespec m_spec;
	pfc::array_t<t_uint8> m_buffer;
	t_size m_samples;
public:
	dsd_chunk_t();
	dsd_chunk_t(const t_uint8* p_buffer, t_size p_samples, t_size p_channels, unsigned p_samplerate, unsigned p_channel_config);
	t_samplespec get_spec();
	t_size get_channels();
	t_size get_sample_count();
	t_size get_size();
	double get_duration();
	pfc::array_t<t_uint8>& get_buffer();
	t_uint8* get_data();
	void set_data(const t_uint8* p_buffer, t_size p_samples, t_size p_channels, unsigned p_samplerate, unsigned p_channel_config);
};

class dsd_stream_service : public service_base {
public:
	virtual bool get_accept_data() = 0;
	virtual void set_accept_data(bool p_accept) = 0;
	virtual t_size get_sample_count() = 0;
	virtual double get_duration() = 0;
	virtual t_samplespec get_spec() = 0;
	virtual bool is_spec_change() = 0;
	virtual void reset_spec_change() = 0;
	virtual dsd_chunk_t* get_first_chunk_ptr() = 0;
	virtual void remove_first_chunk() = 0;
	virtual t_size read(t_uint8* p_buffer, t_size p_samples) = 0;
	virtual void write(const t_uint8* p_buffer, t_size p_samples, t_size p_channels, unsigned p_samplerate, unsigned p_channel_config) = 0;
	virtual void flush() = 0;
	FB2K_MAKE_SERVICE_INTERFACE_ENTRYPOINT(dsd_stream_service);
};

#endif
