/*
* SACD Decoder plugin
* Copyright (c) 2011-2016 Maxim V.Anisiutkin <maxim.anisiutkin@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with FFmpeg; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#ifndef _SACD_WAVPACK_H_INCLUDED
#define _SACD_WAVPACK_H_INCLUDED

#include <minmax.h>
#include <stdint.h>
#include <foobar2000.h>
#include "endianess.h"
#include "scarletbook.h"
#include "sacd_reader.h"
#include "sacd_dsd.h"
#include "wavpack.h"

using namespace pfc;

bool g_dsd_in_wavpack(const char* file);

class sacd_wavpack_t : public sacd_reader_t {
	sacd_media_t*    m_file;
	float            m_dB_gain_adjust;
	WavpackContext*  m_wpc;
	int              m_version;
	uint32_t         m_samplerate;
	int              m_channel_count;
	int              m_channel_mask;
	int              m_loudspeaker_config;
	int              m_framerate;
	int64_t          m_samples_in_stream;
	array_t<uint8_t> m_tags;
	array_t<int32_t> m_data;
public:
	sacd_wavpack_t();
	~sacd_wavpack_t();
	sacd_media_t* get_media();
	uint32_t get_track_count(area_id_e area_id = AREA_BOTH);
	int get_channels();
	int get_loudspeaker_config();
	int get_samplerate();
	int get_framerate();
	uint64_t get_size();
	uint64_t get_offset();
	double get_duration();
	double get_duration(uint32_t subsong);
	bool is_dst();
	bool open(sacd_media_t* p_file, uint32_t mode, float dB_gain_adjust);
	bool close();
	void set_area(area_id_e area_id);
	bool set_track(uint32_t track_number, area_id_e area_id = AREA_BOTH, uint32_t offset = 0);
	bool read_frame(uint8_t* frame_data, size_t* frame_size, frame_type_e* frame_type);
	bool seek(double seconds);
	void get_info(uint32_t subsong, file_info& info);
	void set_info(uint32_t subsong, const file_info& info);
	bool commit();
	bool set_samplerate(sacd_media_t* p_file, uint32_t p_samplerate);
private:
	void adjust_replaygain(replaygain_info& rg_info, float dB_gain_adjust);
	void import_tags(uint8_t wrapper_type, uint8_t* wrapper_data, uint32_t wrapper_size);
};

#endif
